import java.util.Scanner;
import java.io.FileReader;
import java.io.FileNotFoundException;

public class DemoSort {

   int[] getRandomArray(int n) {
      int[] a = new int[n];
      for (int i = 0; i < a.length; ++i) {
         a[i] = (int)(Math.random() * 5.0*n);
      }
      return a;
   }

   int[] copyArray(int[] a) {
      int[] r = a != null ? new int[a.length] : null;
      if (r != null) {
         System.arraycopy( a, 0, r, 0, a.length );
      }
      return r;
   }

   void printArray(int[] array) {
      if (array == null) {
         return;
      }
      for(int i = 0; i < array.length; ++i) {
         System.out.print(array[i] + " ");
      }
      System.out.println();
   }

   void swap(int i, int j, int[] array) {
      int t = array[i];
      array[i] = array[j];
      array[j] = t;
   }

   void selectSort(int[] array) {
      for (int i = 0; i < array.length-1; ++i) {
         int minIDX = i;
         for (int j = i + 1; j < array.length; ++j) {
            if (array[minIDX] > array[j]) {
               minIDX = j;
            }
         }
         swap(minIDX, i, array);
      }
   }

   void qsortPart(int l, int h, int[] array) {
      int i = l; //lower index 
      int j = h; //higher index
      int pivot = array[l + (h - l) / 2];
      while (i <= j) {
         while(array[i] < pivot) {
            i++;
         }
         while(array[j] > pivot) {
            j--;
         }
         if (i <= j) {
            swap(i++, j--, array);
         }
      }
      if (l < j) {
         qsortPart(l, j, array);
      }
      if (i < h) {
         qsortPart(i, h, array);
      }
   }

   void quickSort(int[] array) {
      qsortPart(0, array.length-1, array);
   }


   public void start(int n) {
      int[] values = getRandomArray(n);
      int[] vSelectSort = copyArray(values);
      int[] vQuickSort = copyArray(values);

      long t1 = System.currentTimeMillis();
      System.err.println("Info: start sorting");
      selectSort(vSelectSort);
      long t2 = System.currentTimeMillis();
      System.err.println("Info: select sort done");
      quickSort(vQuickSort);
      long t3 = System.currentTimeMillis();
      System.err.println("Info: quick sort done");
      if (n <= 20) {
         System.out.print("Values: ");
         printArray(values);
         System.out.print("Values Select Sort: ");
         printArray(vSelectSort);
         System.out.print("Values Quick Sort:  ");
         printArray(vQuickSort);
         System.out.println();
      }
      System.out.println("No. of values " + values.length);
      System.out.println("Select Sort required time " + (t2-t1) + " ms");
      System.out.println(" Quick Sort required time " + (t3-t2) + " ms");
   }

   public static void main(String[] args) {
      DemoSort demo = new DemoSort();
      demo.start(args.length > 0 ? Integer.parseInt(args[0]) : 10);
   }

}
